# Standard Library
import datetime as dt

# Third Party
from securegroups.models import SmartFilter, SmartGroup
from securegroups.tasks import run_smart_group_update

# Django
from django.contrib.auth.models import Group
from django.test import TestCase
from django.utils.timezone import now

# Alliance Auth
from allianceauth.eveonline.models import EveCorporationInfo

# Member Audit
from memberaudit.tests.testdata.factories import (
    create_character_corporation_history,
    create_character_title,
)
from memberaudit.tests.testdata.load_entities import load_entities
from memberaudit.tests.utils import create_memberaudit_character

# Alliance Auth (External Libs)
from eveuniverse.models import EveEntity

from .factories import (
    create_corporation_title_filter,
    create_time_in_corporation_filter,
)


class TestFilters(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_entities()
        cls.group = Group.objects.create(name="Leadership")
        cls.corporation_2001 = EveCorporationInfo.objects.get(corporation_id=2001)
        cls.corporation_entity_2001 = EveEntity.objects.get(id=2001)
        cls.corporation_entity_2002 = EveEntity.objects.get(id=2002)

    def test_corporation_title_filter(self):
        # given
        character_1001 = create_memberaudit_character(1001)  # in corp 2001
        user_1001 = character_1001.character_ownership.user
        create_character_title(character=character_1001, name="CEO")

        character_1002 = create_memberaudit_character(1002)  # in corp 2001
        user_1002 = character_1002.character_ownership.user
        create_character_title(character=character_1001, name="Diplomat")

        create_corporation_title_filter(
            corporations=[self.corporation_2001], title="CEO"
        )
        smart_group = SmartGroup.objects.create(group=self.group, auto_group=True)
        title_filter = SmartFilter.objects.first()
        smart_group.filters.add(title_filter)

        # when
        run_smart_group_update(smart_group.id)

        # then
        self.assertEqual(
            title_filter.filter_object.name, "Member Audit Corporation Title"
        )
        self.assertIn(user_1001, self.group.user_set.all())
        self.assertNotIn(user_1002, self.group.user_set.all())

    def test_time_in_corporation_filter(self):
        # given
        character_1001 = create_memberaudit_character(1001)
        user_1001 = character_1001.user
        create_character_corporation_history(
            character=character_1001,
            corporation=self.corporation_entity_2001,
            start_date=now() - dt.timedelta(days=31),
        )

        character_1002 = create_memberaudit_character(1002)
        user_1002 = character_1002.user
        create_character_corporation_history(
            character=character_1002,
            corporation=self.corporation_entity_2001,
            start_date=now() - dt.timedelta(days=29),
        )

        create_time_in_corporation_filter(minimum_days=30)
        smart_group = SmartGroup.objects.create(group=self.group, auto_group=True)
        my_filter = SmartFilter.objects.first()
        smart_group.filters.add(my_filter)

        # when
        run_smart_group_update(smart_group.id)

        # then
        self.assertEqual(
            my_filter.filter_object.name,
            "Member Audit Time in Corporation Filter",
        )
        self.assertIn(user_1001, self.group.user_set.all())
        self.assertNotIn(user_1002, self.group.user_set.all())



"""
The models
"""

# Standard Library
import datetime
from collections import defaultdict
from typing import List

# Third Party
import humanize

# Django
from django.contrib.auth.models import User
from django.core.cache import cache
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.db.models import F, OuterRef, Q, Subquery
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext

# Alliance Auth
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo

# Member Audit
from memberaudit.app_settings import MEMBERAUDIT_APP_NAME
from memberaudit.models import (
    Character,
    CharacterAsset,
    CharacterCorporationHistory,
    CharacterRole,
    CharacterSkillSetCheck,
    CharacterTitle,
    General,
    SkillSet,
)

# Alliance Auth (External Libs)
from eveuniverse.models import EveType


def _get_threshold_date(timedelta_in_days: int) -> datetime.datetime:
    return datetime.datetime.now(datetime.timezone.utc) - datetime.timedelta(
        days=timedelta_in_days
    )


class SingletonModel(models.Model):
    """
    SingletonModel
    """

    class Meta:
        """
        Model meta definitions
        """

        abstract = True

    def delete(self, *args, **kwargs):
        """
        "Delete" action
        :param args:
        :param kwargs:
        :return:
        """

        pass  # pylint: disable=unnecessary-pass

    def set_cache(self):
        """
        Setting cache
        :return:
        """

        cache.set(self.__class__.__name__, self)

    def save(self, *args, **kwargs):
        """
        "Save" action
        :param args:
        :param kwargs:
        :return:
        """

        self.pk = 1
        super().save(*args, **kwargs)

        self.set_cache()

    @classmethod
    def load(cls):
        """
        Get cache
        :return:
        """

        if cache.get(cls.__name__) is None:
            obj, created = cls.objects.get_or_create(pk=1)

            if not created:
                obj.set_cache()

        return cache.get(cls.__name__)


class BaseFilter(models.Model):
    """
    BaseFilter
    """

    description = models.CharField(
        max_length=500,
        help_text=_("The filter description that is shown to end users."),
    )  # this is what is shown to the user

    class Meta:
        """
        Model meta definitions
        """

        abstract = True

    def __str__(self):
        """
        Model stringified name
        :return:
        """

        return f"{self.name}: {self.description}"

    def process_filter(self, user: User):
        """
        This is the check run against a users characters
        :param user:
        :return:
        """

        raise NotImplementedError(_("Please create a filter!"))

    def audit_filter(self, users: models.QuerySet[User]) -> dict:
        """
        Return information for each given user wether they passes the filter
        and a help message shown in the audit feature.
        """

        raise NotImplementedError(_("Please create an audit function!"))


class ActivityFilter(BaseFilter):
    """
    ActivityFilter
    """

    inactivity_threshold = models.PositiveIntegerField(
        help_text=_("Maximum allowable inactivity, in <strong>days</strong>.")
    )

    @property
    def name(self):
        """
        Filter name
        :return:
        """

        inactivity_threshold = ngettext(
            f"{self.inactivity_threshold:d} day",
            f"{self.inactivity_threshold:d} days",
            self.inactivity_threshold,
        )

        return _(f"Activity [Last {inactivity_threshold}]")

    def process_filter(self, user: User):
        """
        Processing filter
        :param user:
        :return:
        """

        threshold_date = _get_threshold_date(
            timedelta_in_days=self.inactivity_threshold
        )

        return (
            Character.objects.owned_by_user(user=user)
            .filter(
                Q(online_status__last_login__gt=threshold_date)
                | Q(online_status__last_logout__gt=threshold_date),
            )
            .count()
            > 0
        )

    def audit_filter(self, users):
        """
        Audit Filter
        :param users:
        :type users:
        :return:
        :rtype:
        """

        threshold_date = _get_threshold_date(
            timedelta_in_days=self.inactivity_threshold
        )

        output = defaultdict(lambda: {"message": "", "check": False})

        for user in users:
            characters = Character.objects.owned_by_user(user=user).filter(
                Q(online_status__last_login__gt=threshold_date)
                | Q(online_status__last_logout__gt=threshold_date),
            )

            if characters.count() > 0:
                chars = defaultdict(list)

                for char in characters:
                    chars[char.user.pk].append(char.eve_character.character_name)

                for char_user, char_list in chars.items():
                    message = ngettext(
                        "Active character: ", "Active characters: ", len(char_list)
                    )

                    output[char_user] = {
                        "message": message + ", ".join(sorted(char_list)),
                        "check": True,
                    }

        return output


class AgeFilter(BaseFilter):
    """
    AgeFilter
    """

    age_threshold = models.PositiveIntegerField(
        help_text=_("Minimum allowable age, in <strong>days</strong>.")
    )

    @property
    def name(self):
        """
        Filter name
        :return:
        """

        age_threshold = ngettext(
            f"{self.age_threshold:d} day",
            f"{self.age_threshold:d} days",
            self.age_threshold,
        )

        return _(f"Character age [{age_threshold}]")

    def process_filter(self, user: User):
        """
        Processing filter
        :param user:
        :return:
        """

        threshold_date = _get_threshold_date(timedelta_in_days=self.age_threshold)

        return (
            Character.objects.owned_by_user(user=user)
            .filter(details__birthday__lt=threshold_date)
            .count()
            > 0
        )

    def audit_filter(self, users):
        """
        Audit Filter
        :param users:
        :type users:
        :return:
        :rtype:
        """

        threshold_date = _get_threshold_date(timedelta_in_days=self.age_threshold)

        output = defaultdict(lambda: {"message": "", "check": False})

        for user in users:
            characters = Character.objects.owned_by_user(user=user).filter(
                details__birthday__lt=threshold_date
            )

            if characters.count() > 0:
                chars = defaultdict(list)

                for char in characters:
                    chars[char.user.pk].append(char.eve_character.character_name)

                for char_user, char_list in chars.items():
                    output[char_user] = {
                        "message": ", ".join(sorted(char_list)),
                        "check": True,
                    }

        return output


class AssetFilter(BaseFilter):
    """
    AssetFilter
    """

    assets = models.ManyToManyField(
        EveType,
        help_text=_("User must possess <strong>one</strong> of the selected assets."),
    )

    @property
    def name(self):
        """
        Filter name
        :return:
        """

        return _("Member Audit Asset")

    def process_filter(self, user: User):
        """
        Processing filter
        :param user:
        :return:
        """

        return CharacterAsset.objects.filter(
            character__eve_character__character_ownership__user=user,
            eve_type__in=self.assets.all(),
        ).exists()

    def audit_filter(self, users: models.QuerySet[User]):
        """
        Audit Filter
        """

        matching_assets = CharacterAsset.objects.filter(
            character=OuterRef("pk"), eve_type__in=list(self.assets.all())
        )
        characters = (
            Character.objects.filter(
                eve_character__character_ownership__user__in=list(users)
            )
            .annotate(asset_name=Subquery(matching_assets.values("eve_type__name")[:1]))
            .values(
                "asset_name",
                user_id=F("eve_character__character_ownership__user_id"),
                character_name=F("eve_character__character_name"),
            )
        )

        output_users = {}
        for character in characters:
            user_id = character["user_id"]
            if user_id not in output_users:
                output_users[user_id] = []

            asset_name = character["asset_name"]
            if asset_name:
                character_name = character["character_name"]
                output_users[character["user_id"]].append(
                    f"{character_name} ({asset_name})"
                )

        output = {}
        for user_id, matches in output_users.items():
            if matches:
                message = ", ".join(sorted(matches))
                check = True
            else:
                message = "No matching assets"
                check = False

            output[user_id] = {"message": message, "check": check}

        user_ids = set(users.values_list("id", flat=True))
        missing_user_ids = user_ids - set(output.keys())
        for user_id in missing_user_ids:
            output[user_id] = {"message": "No audit info", "check": False}

        return output


class ComplianceFilter(BaseFilter, SingletonModel):
    """
    ComplianceFilter
    """

    @property
    def name(self):
        """
        Filter name
        :return:
        """

        return _("Compliance")

    def process_filter(self, user: User) -> bool:
        """
        Return True if the user is compliant, else False.
        :param user:
        :type user:
        :return:
        :rtype:
        """

        unregistered_characters = EveCharacter.objects.filter(
            character_ownership__user=user, memberaudit_character__isnull=True
        )

        return not unregistered_characters.exists()

    def audit_filter(self, users) -> dict:
        """
        Return audit data for compliance of given users.
        :param users:
        :type users:
        :return:
        :rtype:
        """

        unregistered_characters = EveCharacter.objects.filter(
            character_ownership__user__in=list(users),
            memberaudit_character__isnull=True,
        ).values("character_name", user_id=F("character_ownership__user_id"))

        user_with_unregistered_characters = defaultdict(list)

        for obj in unregistered_characters:
            character_name = obj["character_name"]
            user_with_unregistered_characters[obj["user_id"]].append(
                f"{character_name}"
            )

        all_memberaudit_users_ids = General.users_with_basic_access().values_list(
            "id", flat=True
        )

        output = {}

        for user_id in all_memberaudit_users_ids:
            unregistered_chars = user_with_unregistered_characters.get(user_id)

            if not unregistered_chars:
                output[user_id] = {
                    "message": _(
                        f"All characters have been added to {MEMBERAUDIT_APP_NAME}"
                    ),
                    "check": True,
                }
            else:
                missing_characters_message = ngettext(
                    "Missing character: ",
                    "Missing characters: ",
                    len(unregistered_chars),
                )
                output[user_id] = {
                    "message": missing_characters_message
                    + ", ".join(sorted(unregistered_chars)),
                    "check": False,
                }

        return output


class CorporationRoleFilter(BaseFilter):
    """
    Filter for corporation roles.
    """

    corporations = models.ManyToManyField(
        EveCorporationInfo,
        related_name="+",
        help_text=_(
            "The character with the role must be in one of these corporations."
        ),
    )
    role = models.CharField(
        max_length=3,
        choices=CharacterRole.Role.choices,
        db_index=True,
        help_text=_("User must have a character with this role."),
    )
    include_alts = models.BooleanField(
        default=False,
        help_text=_(
            "When True, the filter will also include the users alt-characters."
        ),
    )

    @property
    def name(self):
        """
        Return name of this filter.
        """

        return _("Member Audit Corporation Role")

    def process_filter(self, user: User) -> bool:
        """
        Return True when filter applies to the user, else False.
        """

        qs = CharacterRole.objects.filter(
            character__eve_character__character_ownership__user=user,
            character__eve_character__corporation_id__in=self._corporation_ids(),
            role=self.role,
            location=CharacterRole.Location.UNIVERSAL,
        )

        if not self.include_alts:
            qs = qs.filter(character__eve_character__userprofile__isnull=False)

        return qs.exists()

    def audit_filter(self, users):
        qs = Character.objects.filter(
            eve_character__character_ownership__user__in=list(users),
            eve_character__corporation_id__in=self._corporation_ids(),
            roles__role=self.role,
            roles__location=(CharacterRole.Location.UNIVERSAL),
        )

        if not self.include_alts:
            qs = qs.filter(eve_character__userprofile__isnull=False)

        matching_characters = qs.values(
            user_id=F("eve_character__character_ownership__user_id"),
            character_name=F("eve_character__character_name"),
        )

        user_with_characters = defaultdict(list)
        for user_id in matching_characters:
            character_name = user_id["character_name"]
            user_with_characters[user_id["user_id"]].append(f"{character_name}")

        output = {
            user_id: {"message": "No matching character", "check": False}
            for user_id in users.values_list("id", flat=True)
        }

        for user_id, character_names in user_with_characters.items():
            output[user_id] = {
                "message": ", ".join(sorted(character_names)),
                "check": True,
            }

        return output

    def _corporation_ids(self) -> List[int]:
        """
        Return Eve IDs of corporations in this filter.
        """

        return list(self.corporations.values_list("corporation_id", flat=True))


class CorporationTitleFilter(BaseFilter):
    """
    Filter for corporation titles.
    """

    corporations = models.ManyToManyField(
        EveCorporationInfo,
        related_name="+",
        help_text=_(
            "The character with the title must be in one of these corporations."
        ),
    )
    title = models.CharField(
        max_length=100,
        db_index=True,
        help_text=_("User must have a character with this title."),
    )
    include_alts = models.BooleanField(
        default=False,
        help_text=_(
            "When True, the filter will also include the users alt-characters."
        ),
    )

    @property
    def name(self):
        """
        Return name of this filter.
        """

        return _("Member Audit Corporation Title")

    def process_filter(self, user: User) -> bool:
        """
        Return True when filter applies to the user, else False.
        """

        qs = CharacterTitle.objects.filter(
            character__eve_character__character_ownership__user=user,
            character__eve_character__corporation_id__in=self._corporation_ids(),
            name=self.title,
        )

        if not self.include_alts:
            qs = qs.filter(character__eve_character__userprofile__isnull=False)

        return qs.exists()

    def audit_filter(self, users):
        qs = Character.objects.filter(
            eve_character__character_ownership__user__in=list(users),
            eve_character__corporation_id__in=self._corporation_ids(),
            titles__name=self.title,
        )

        if not self.include_alts:
            qs = qs.filter(eve_character__userprofile__isnull=False)

        matching_characters = qs.values(
            user_id=F("eve_character__character_ownership__user_id"),
            character_name=F("eve_character__character_name"),
        )

        user_with_characters = defaultdict(list)
        for user_id in matching_characters:
            character_name = user_id["character_name"]
            user_with_characters[user_id["user_id"]].append(f"{character_name}")

        output = {
            user_id: {"message": "No matching character", "check": False}
            for user_id in users.values_list("id", flat=True)
        }

        for user_id, character_names in user_with_characters.items():
            output[user_id] = {
                "message": ", ".join(sorted(character_names)),
                "check": True,
            }

        return output

    def _corporation_ids(self) -> List[int]:
        """
        Return Eve IDs of corporations in this filter.
        """

        return list(self.corporations.values_list("corporation_id", flat=True))


class SkillPointFilter(BaseFilter):
    """
    SkillPointFilter
    """

    skill_point_threshold = models.PositiveBigIntegerField(
        help_text=_("Minimum allowable skill points.")
    )

    @property
    def name(self):
        """
        Filter name
        :return:
        """

        sp_threshold = humanize.intword(self.skill_point_threshold)

        skill_point_threshold = ngettext(
            f"{sp_threshold} skill point",
            f"{sp_threshold} skill points",
            self.skill_point_threshold,
        )

        return _(f"Member Audit Skill Points [{skill_point_threshold}]")

    def process_filter(self, user: User):
        """
        Processing filter
        :param user:
        :return:
        """

        return (
            Character.objects.owned_by_user(user=user)
            .filter(skillpoints__total__gt=self.skill_point_threshold)
            .count()
            > 0
        )

    def audit_filter(self, users):
        """
        Audit Filter
        :param users:
        :type users:
        :return:
        :rtype:
        """

        output = defaultdict(lambda: {"message": "", "check": False})

        for user in users:
            characters = Character.objects.owned_by_user(user=user).filter(
                skillpoints__total__gt=self.skill_point_threshold
            )

            if characters.count() > 0:
                chars = defaultdict(list)

                for char in characters:
                    chars[char.user.pk].append(char.eve_character.character_name)

                for char_user, char_list in chars.items():
                    output[char_user] = {
                        "message": ", ".join(sorted(char_list)),
                        "check": True,
                    }

        return output


class SkillSetFilter(BaseFilter):
    """
    SkillSetFilter
    """

    class CharacterType(models.TextChoices):
        """
        A character type.
        """

        ANY = "AN", _("Any")
        MAINS_ONLY = "MO", _("Mains only")
        ALTS_ONLY = "AO", _("Alts only")

    skill_sets = models.ManyToManyField(
        SkillSet,
        help_text=_(
            "Users must have a character who possess all of the skills in "
            "<strong>one</strong> of the selected skill sets."
        ),
    )
    character_type = models.CharField(
        max_length=2,
        choices=CharacterType.choices,
        default=CharacterType.ANY,
        blank=True,
        help_text=_("Specify the type of character that needs to have the skill set."),
    )

    def save(
        self,
        force_insert=False,  # pylint: disable=unused-argument
        force_update=False,  # pylint: disable=unused-argument
        using=None,  # pylint: disable=unused-argument
        update_fields=None,  # pylint: disable=unused-argument
    ):
        """
        Saving action

        :param force_insert:
        :type force_insert:
        :param force_update:
        :type force_update:
        :param using:
        :type using:
        :param update_fields:
        :type update_fields:
        :return:
        :rtype:
        """

        # Make sure a character_type is set
        if self.character_type == "":
            self.character_type = self.CharacterType.ANY

        super().save()

    @property
    def name(self):
        """
        Filter name
        :return:
        """

        return _("Member Audit Skill Set")

    def process_filter(self, user: User):
        """
        Processing filter
        :param user:
        :return:
        """

        qs = CharacterSkillSetCheck.objects.filter(
            character__eve_character__character_ownership__user=user,
            skill_set__in=list(self.skill_sets.all()),
            failed_required_skills__isnull=True,
        )

        if self.character_type == self.CharacterType.MAINS_ONLY:
            qs = qs.filter(character__eve_character__userprofile__isnull=False)
        elif self.character_type == self.CharacterType.ALTS_ONLY:
            qs = qs.filter(character__eve_character__userprofile__isnull=True)

        return qs.exists()

    def audit_filter(self, users):
        """
        Audit Filter
        :param users:
        :type users:
        :return:
        :rtype:
        """

        qs = Character.objects.filter(
            skill_set_checks__skill_set__in=list(self.skill_sets.all()),
            skill_set_checks__failed_required_skills__isnull=True,
        )

        if self.character_type == self.CharacterType.MAINS_ONLY:
            qs = qs.filter(eve_character__userprofile__isnull=False)
        elif self.character_type == self.CharacterType.ALTS_ONLY:
            qs = qs.filter(eve_character__userprofile__isnull=True)

        matching_characters = qs.values(
            user_id=F("eve_character__character_ownership__user_id"),
            character_name=F("eve_character__character_name"),
        )

        user_with_characters = defaultdict(list)

        for user_id in matching_characters:
            character_name = user_id["character_name"]
            user_with_characters[user_id["user_id"]].append(f"{character_name}")

        output = {
            user_id: {"message": "No matching character", "check": False}
            for user_id in users.values_list("id", flat=True)
        }

        for user_id, character_names in user_with_characters.items():
            output[user_id] = {
                "message": ", ".join(sorted(character_names)),
                "check": True,
            }

        return output


class TimeInCorporationFilter(BaseFilter):
    minimum_days = models.PositiveIntegerField(
        default=30,
        help_text=(
            "Minimum number of days a main character needs to be member "
            "of his/her current corporation."
        ),
    )

    @property
    def name(self):
        """
        Return name of this filter.
        """

        return _("Member Audit Time in Corporation Filter")

    def process_filter(self, user: User) -> bool:
        """
        Return True when filter applies to the user, else False.
        """

        try:
            character = user.profile.main_character.memberaudit_character
        except (ObjectDoesNotExist, AttributeError):
            return False

        history = (
            character.corporation_history.exclude(is_deleted=True)
            .order_by("-record_id")
            .first()
        )
        if not history:
            return False

        passes = (now() - history.start_date).days >= self.minimum_days
        return passes

    def audit_filter(self, users) -> dict:
        """
        Return result of filter audit for given users.
        """

        current_membership = (
            CharacterCorporationHistory.objects.filter(
                character=OuterRef("profile__main_character__memberaudit_character__pk")
            )
            .exclude(is_deleted=True)
            .order_by("-record_id")
        )
        users_days_in_corporation = users.annotate(
            start_date=Subquery(current_membership.values("start_date")[:1])
        )

        output = defaultdict(lambda: {"message": "", "check": False})
        for user in users_days_in_corporation:
            if not user.start_date:
                check = False
                msg = "No audit info"
            else:
                days_in_corporation = (now() - user.start_date).days
                check = days_in_corporation >= self.minimum_days
                msg = f"{days_in_corporation} days"

            output[user.id] = {"message": msg, "check": check}

        return output



# Django
from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse

# Alliance Auth
from allianceauth.eveonline.models import EveCorporationInfo

# Member Audit
from memberaudit.models import CharacterRole
from memberaudit.tests.testdata.load_entities import load_entities

# Memberaudit Securegroups
from memberaudit_securegroups.models import (
    CorporationRoleFilter,
    CorporationTitleFilter,
    TimeInCorporationFilter,
)


class TestCorporationRoleFilter(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_entities()
        cls.corporation_2001 = EveCorporationInfo.objects.get(corporation_id=2001)
        cls.corporation_2101 = EveCorporationInfo.objects.get(corporation_id=2101)
        cls.user = User.objects.create(
            username="superman", is_staff=True, is_superuser=True
        )
        cls.admin_add_url = reverse(
            "admin:memberaudit_securegroups_corporationrolefilter_add"
        )

    def test_should_create_new_filter(self):
        # given
        self.client.force_login(self.user)
        data = {
            "description": "dummy",
            "role": CharacterRole.Role.DIRECTOR,
            "corporations": f"{self.corporation_2001.id}",
        }
        # when
        response = self.client.post(self.admin_add_url, data)
        # then
        self.assertEqual(response.status_code, 302)
        obj = CorporationRoleFilter.objects.first()
        self.assertEqual(obj.role, CharacterRole.Role.DIRECTOR)
        self.assertEqual(
            {2001}, set(obj.corporations.values_list("corporation_id", flat=True))
        )

    def test_should_not_allow_creating_filter_without_defining_at_least_one_corporation(
        self,
    ):
        # given
        self.client.force_login(self.user)
        data = {"description": "dummy", "role": CharacterRole.Role.DIRECTOR}
        # when
        response = self.client.post(self.admin_add_url, data)
        # then
        self.assertEqual(response.status_code, 200)


class TestCorporationTitleFilter(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_entities()
        cls.corporation_2001 = EveCorporationInfo.objects.get(corporation_id=2001)
        cls.corporation_2101 = EveCorporationInfo.objects.get(corporation_id=2101)
        cls.user = User.objects.create(
            username="superman", is_staff=True, is_superuser=True
        )
        cls.admin_add_url = reverse(
            "admin:memberaudit_securegroups_corporationtitlefilter_add"
        )

    def test_should_create_new_filter(self):
        # given
        self.client.force_login(self.user)
        data = {
            "description": "dummy",
            "title": "Alpha",
            "corporations": f"{self.corporation_2001.id}",
        }
        # when
        response = self.client.post(self.admin_add_url, data)
        # then
        self.assertEqual(response.status_code, 302)
        obj = CorporationTitleFilter.objects.first()
        self.assertEqual(obj.title, "Alpha")
        self.assertEqual(
            {2001}, set(obj.corporations.values_list("corporation_id", flat=True))
        )

    def test_should_not_allow_creating_filter_without_defining_at_least_one_corporation(
        self,
    ):
        # given
        self.client.force_login(self.user)
        data = {"description": "dummy", "title": "Alpha"}
        # when
        response = self.client.post(self.admin_add_url, data)
        # then
        self.assertEqual(response.status_code, 200)


class TestTimeInCorpFilter(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.user = User.objects.create(
            username="superman", is_staff=True, is_superuser=True
        )
        cls.admin_add_url = reverse(
            "admin:memberaudit_securegroups_timeincorporationfilter_add"
        )

    def test_should_create_new_filter(self):
        # given
        self.client.force_login(self.user)
        data = {
            "description": "dummy",
            "minimum_days": 45,
        }
        # when
        response = self.client.post(self.admin_add_url, data)
        # then
        self.assertEqual(response.status_code, 302)
        obj = TimeInCorporationFilter.objects.first()
        self.assertEqual(obj.minimum_days, 45)



"""
App's base definitions
"""

# Django
from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _

# Memberaudit Securegroups
from memberaudit_securegroups import __version__


class MemberAuditSecureGroupsConfig(AppConfig):
    """
    MemberAuditSecureGroupsConfig
    """

    name = "memberaudit_securegroups"
    label = "memberaudit_securegroups"
    verbose_name = _(f"Secure Groups Member Audit Integration v{__version__}")



"""
Admin pages
"""

# Standard Library
from typing import Any

# Third Party
import humanize

# Django
from django.contrib import admin
from django.db.models.query import QuerySet
from django.http.request import HttpRequest
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext

# Member Audit
from memberaudit.models import CharacterRole

# Memberaudit Securegroups
from memberaudit_securegroups.models import (
    ActivityFilter,
    AgeFilter,
    AssetFilter,
    ComplianceFilter,
    CorporationRoleFilter,
    CorporationTitleFilter,
    SkillPointFilter,
    SkillSetFilter,
    TimeInCorporationFilter,
)


class SingletonModelAdmin(admin.ModelAdmin):
    """
    Prevents Django admin users deleting the singleton or adding extra rows.
    """

    actions = None  # Removes the default delete action.

    def has_delete_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return self.model.objects.all().count() == 0


@admin.register(ActivityFilter)
class ActivityFilterAdmin(admin.ModelAdmin):
    """
    ActivityFilterAdmin
    """

    list_display = ("description", "_inactivity_threshold")

    def _inactivity_threshold(self, obj):
        inactivity_threshold = obj.inactivity_threshold

        return_value = ngettext(
            f"{inactivity_threshold:d} day",
            f"{inactivity_threshold:d} days",
            inactivity_threshold,
        )

        return return_value


@admin.register(AgeFilter)
class AgeFilterAdmin(admin.ModelAdmin):
    """
    AgeFilterAdmin
    """

    list_display = ("description", "_age_threshold")

    def _age_threshold(self, obj):
        return f"{obj.age_threshold:d} days"


@admin.register(AssetFilter)
class AssetFilterAdmin(admin.ModelAdmin):
    """
    AssetFilterAdmin
    """

    list_display = ("description", "_assets")
    autocomplete_fields = ["assets"]

    def get_queryset(self, request: HttpRequest) -> QuerySet[Any]:
        qs = super().get_queryset(request)

        return qs.prefetch_related("assets")

    @admin.display()
    def _assets(self, obj) -> str:
        objs = obj.assets.all()

        return ", ".join(sorted([obj.name for obj in objs]))


@admin.register(ComplianceFilter)
class ComplianceFilterAdmin(SingletonModelAdmin):
    """
    ComplianceFilterAdmin
    """

    list_display = ("description",)


class CorporationRoleListFilter(admin.SimpleListFilter):
    title = _("corporation role")
    parameter_name = "corporation_role"

    def lookups(self, request, model_admin):
        """
        Return lookups with used roles only.
        """

        roles = set(model_admin.get_queryset(request).values_list("role", flat=True))
        result = [(role, CharacterRole.Role(role).label) for role in roles]

        return sorted(result, key=lambda o: o[1])

    def queryset(self, request, queryset):
        """
        Return queryset for selected role.
        """

        if value := self.value():
            return queryset.filter(role=value)


@admin.register(CorporationRoleFilter)
class CorporationRoleFilterAdmin(admin.ModelAdmin):
    """
    CorporationRoleFilterAdmin
    """

    list_display = ("description", "role", "_corporations")
    list_filter = (CorporationRoleListFilter,)
    filter_horizontal = ("corporations",)
    fields = ("description", "role", "corporations", "include_alts")

    def get_queryset(self, request: HttpRequest) -> QuerySet[Any]:
        qs = super().get_queryset(request)

        return qs.prefetch_related("corporations")

    @admin.display()
    def _corporations(self, obj) -> str:
        objs = obj.corporations.all()

        return ", ".join(sorted([obj.corporation_name for obj in objs]))


@admin.register(CorporationTitleFilter)
class CorporationTitleFilterAdmin(admin.ModelAdmin):
    """
    CorporationTitleFilterAdmin
    """

    list_display = ("description", "title", "_corporations")
    filter_horizontal = ("corporations",)
    fields = ("description", "title", "corporations", "include_alts")

    def get_queryset(self, request: HttpRequest) -> QuerySet[Any]:
        qs = super().get_queryset(request)

        return qs.prefetch_related("corporations")

    @admin.display()
    def _corporations(self, obj) -> str:
        objs = obj.corporations.all()

        return ", ".join(sorted([obj.corporation_name for obj in objs]))


@admin.register(SkillPointFilter)
class SkillPointFilterAdmin(admin.ModelAdmin):
    """
    SkillPointFilterAdmin
    """

    list_display = ("description", "_skill_point_threshold")

    def _skill_point_threshold(self, obj):
        skillpoints = humanize.intword(obj.skill_point_threshold)

        return f"{skillpoints} skill points"


@admin.register(SkillSetFilter)
class SkillSetFilterAdmin(admin.ModelAdmin):
    """
    SkillSetFilterAdmin
    """

    list_display = ("description", "_skill_sets", "character_type")
    filter_horizontal = ("skill_sets",)

    def get_queryset(self, request: HttpRequest) -> QuerySet[Any]:
        qs = super().get_queryset(request)

        return qs.prefetch_related("skill_sets")

    @admin.display()
    def _skill_sets(self, obj) -> str:
        objs = obj.skill_sets.all()

        return ", ".join(sorted([obj.name for obj in objs]))


@admin.register(TimeInCorporationFilter)
class TimeInCorporationFilterAdmin(admin.ModelAdmin):
    """
    TimeInCorporationFilterAdmin
    """

    list_display = ("description", "minimum_days")



# Standard Library
from typing import Iterable

# Alliance Auth
from allianceauth.eveonline.models import EveCorporationInfo

# Member Audit
from memberaudit.models import CharacterRole

# Memberaudit Securegroups
from memberaudit_securegroups.models import (
    CorporationRoleFilter,
    CorporationTitleFilter,
    TimeInCorporationFilter,
)


def create_corporation_role_filter(
    corporations: Iterable[EveCorporationInfo], **kwargs
) -> CorporationRoleFilter:
    params = {"role": CharacterRole.Role.DIRECTOR}
    params.update(kwargs)
    obj: CorporationRoleFilter = CorporationRoleFilter.objects.create(**params)
    if corporations:
        obj.corporations.add(*list(corporations))
    return obj


def create_corporation_title_filter(
    corporations: Iterable[EveCorporationInfo], **kwargs
) -> CorporationTitleFilter:
    params = {"title": "title"}
    params.update(kwargs)
    obj: CorporationTitleFilter = CorporationTitleFilter.objects.create(**params)
    if corporations:
        obj.corporations.add(*list(corporations))
    return obj


def create_time_in_corporation_filter(
    **kwargs,
) -> TimeInCorporationFilter:
    params = {"minimum_days": 30}
    params.update(kwargs)
    obj: TimeInCorporationFilter = TimeInCorporationFilter.objects.create(**params)
    return obj



# Generated by Django 4.2.10 on 2024-02-11 11:09

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eveonline", "0017_alliance_and_corp_names_are_not_unique"),
        ("memberaudit_securegroups", "0003_add_corporation_role_filter_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="TimeInCorporationFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        help_text="The filter description that is shown to end users.",
                        max_length=500,
                    ),
                ),
                (
                    "minimum_days",
                    models.PositiveIntegerField(
                        default=30,
                        help_text="Minimum number of days a main character needs to be member of his/her current corporation.",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="CorporationTitleFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        help_text="The filter description that is shown to end users.",
                        max_length=500,
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        db_index=True,
                        help_text="User must have a character with this title.",
                        max_length=100,
                    ),
                ),
                (
                    "include_alts",
                    models.BooleanField(
                        default=False,
                        help_text="When True, the filter will also include the users alt-characters.",
                    ),
                ),
                (
                    "corporations",
                    models.ManyToManyField(
                        help_text="The character with the title must be in one of these corporations.",
                        related_name="+",
                        to="eveonline.evecorporationinfo",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



# Generated by Django 4.0.10 on 2023-09-22 21:50

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("memberaudit", "0011_add_standings_index"),
        ("memberaudit_securegroups", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="skillpointfilter",
            name="skill_point_threshold",
            field=models.PositiveBigIntegerField(
                help_text="Minimum allowable skill points."
            ),
        ),
        migrations.AlterField(
            model_name="skillsetfilter",
            name="skill_sets",
            field=models.ManyToManyField(
                help_text="Users must possess all of the skills in <strong>one</strong> of the selected skill sets.",
                to="memberaudit.skillset",
            ),
        ),
    ]



# Standard Library
import datetime as dt

# Django
from django.contrib.auth.models import User
from django.test import TestCase
from django.utils.timezone import now

# Alliance Auth
from allianceauth.eveonline.models import EveCorporationInfo

# Member Audit
from memberaudit.app_settings import MEMBERAUDIT_APP_NAME
from memberaudit.models import CharacterRole
from memberaudit.tests.testdata.factories import (
    create_character_asset,
    create_character_corporation_history,
    create_character_role,
    create_character_skill_set_check,
    create_character_title,
    create_skill_set,
    create_skill_set_skill,
)
from memberaudit.tests.testdata.load_entities import load_entities
from memberaudit.tests.testdata.load_eveuniverse import load_eveuniverse
from memberaudit.tests.utils import (
    add_auth_character_to_user,
    add_memberaudit_character_to_user,
    create_memberaudit_character,
    create_user_from_evecharacter_with_access,
)

# Alliance Auth (External Libs)
from eveuniverse.models import EveEntity, EveType

# Memberaudit Securegroups
from memberaudit_securegroups.models import (
    AssetFilter,
    ComplianceFilter,
    SkillSetFilter,
)

from .factories import (
    create_corporation_role_filter,
    create_corporation_title_filter,
    create_time_in_corporation_filter,
)


class TestAssetFilter(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_entities()
        load_eveuniverse()
        cls.character_1001 = create_memberaudit_character(1001)
        cls.user_1 = cls.character_1001.character_ownership.user

    def test_should_return_name(self):
        # given
        my_filter = AssetFilter.objects.create()
        # when/then
        self.assertTrue(my_filter.name)

    def test_should_return_false_when_user_does_not_have_asset(self):
        # given
        my_filter = AssetFilter.objects.create()
        merlin_type = EveType.objects.get(name="Merlin")
        my_filter.assets.add(merlin_type)
        astrahus_type = EveType.objects.get(name="Astrahus")
        create_character_asset(character=self.character_1001, eve_type=astrahus_type)
        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_true_when_user_has_asset(self):
        # given
        my_filter = AssetFilter.objects.create()
        merlin_type = EveType.objects.get(name="Merlin")
        my_filter.assets.add(merlin_type)
        create_character_asset(character=self.character_1001, eve_type=merlin_type)
        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))

    def test_should_return_true_when_user_has_at_least_one_asset(self):
        # given
        my_filter = AssetFilter.objects.create()
        merlin_type = EveType.objects.get(name="Merlin")
        astrahus_type = EveType.objects.get(name="Astrahus")
        my_filter.assets.add(merlin_type, astrahus_type)
        create_character_asset(character=self.character_1001, eve_type=merlin_type)
        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))

    def test_should_return_audit_data_for_one_matching_one_not_matching_user(self):
        # given a filter for Merlins
        my_filter = AssetFilter.objects.create()
        merlin_type = EveType.objects.get(name="Merlin")
        my_filter.assets.add(merlin_type)

        # and main user's character has a Merlin
        create_character_asset(character=self.character_1001, eve_type=merlin_type)

        # and main user's 2nd character also has a Merlin
        character_1002 = add_memberaudit_character_to_user(self.user_1, 1002)
        create_character_asset(character=character_1002, eve_type=merlin_type)

        # and a 2nd user has a registered character, but no Merlin
        character_1003 = create_memberaudit_character(1003)
        user_1002 = character_1003.character_ownership.user
        # and a 3rd user is not registered
        user_2, _ = create_user_from_evecharacter_with_access(1101)

        # when
        users = make_user_queryset(self.user_1, user_1002, user_2)
        result = my_filter.audit_filter(users)

        # then
        self.assertDictEqual(
            result[self.user_1.id],
            {"message": "Bruce Wayne (Merlin), Clark Kent (Merlin)", "check": True},
        )
        self.assertDictEqual(
            result[user_1002.id], {"message": "No matching assets", "check": False}
        )
        self.assertDictEqual(
            result[user_2.id], {"message": "No audit info", "check": False}
        )
        self.assertEqual(len(result), 3)

    def test_should_return_audit_data_when_no_matches(self):
        # given
        my_filter = AssetFilter.objects.create()
        # when
        users = make_user_queryset(self.user_1)
        result = my_filter.audit_filter(users)
        # then
        self.assertEqual(len(result), 1)
        self.assertDictEqual(
            result[self.user_1.id], {"message": "No matching assets", "check": False}
        )


class TestComplianceFilter(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_entities()
        cls.user_1, _ = create_user_from_evecharacter_with_access(1001)

    def test_should_return_name(self):
        # given
        my_filter = ComplianceFilter.objects.create()
        # when/then
        self.assertTrue(my_filter.name)

    def test_should_return_true_when_user_is_compliant_1(self):
        # given a user with 1 registered character
        add_memberaudit_character_to_user(self.user_1, 1001)
        my_filter = ComplianceFilter.objects.create()
        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))

    def test_should_return_true_when_user_is_compliant_2(self):
        # given a user with 2 registered character
        add_memberaudit_character_to_user(self.user_1, 1001)
        add_memberaudit_character_to_user(self.user_1, 1002)
        my_filter = ComplianceFilter.objects.create()
        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))

    def test_should_return_false_when_user_is_not_compliant_1(self):
        # given a user with 1 unregistered character
        my_filter = ComplianceFilter.objects.create()
        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_false_when_user_is_not_compliant_2(self):
        # given a user with 1 registered and 1 unregistered character
        add_memberaudit_character_to_user(self.user_1, 1001)
        add_auth_character_to_user(self.user_1, 1002)
        my_filter = ComplianceFilter.objects.create()
        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_audit_data_for_users(self):
        # given
        my_filter = ComplianceFilter.objects.create()
        add_memberaudit_character_to_user(self.user_1, 1001)
        user_2, _ = create_user_from_evecharacter_with_access(1101)
        # when
        users = make_user_queryset(self.user_1, user_2)
        result = my_filter.audit_filter(users)
        # then
        self.assertEqual(len(result), 2)
        result_user_1001 = result[self.user_1.pk]
        self.assertTrue(result_user_1001["check"])
        self.assertEqual(
            result_user_1001["message"],
            f"All characters have been added to {MEMBERAUDIT_APP_NAME}",
        )
        result_user_1101 = result[user_2.pk]
        self.assertFalse(result_user_1101["check"])
        self.assertIn("Lex Luther", result_user_1101["message"])

    def test_should_return_audit_data_for_non_compliant_user_with_1_character(self):
        # given
        my_filter = ComplianceFilter.objects.create()

        # when
        users = make_user_queryset(self.user_1)
        result = my_filter.audit_filter(users)

        # then
        expected = {
            self.user_1.pk: {
                "check": False,
                "message": "Missing character: Bruce Wayne",
            },
        }
        self.assertDictEqual(result, expected)

    def test_should_return_audit_data_for_non_compliant_user_with_2_characters(self):
        # given
        add_auth_character_to_user(self.user_1, 1002)
        my_filter = ComplianceFilter.objects.create()

        # when
        users = make_user_queryset(self.user_1)
        result = my_filter.audit_filter(users)

        # then
        expected = {
            self.user_1.pk: {
                "check": False,
                "message": "Missing characters: Bruce Wayne, Clark Kent",
            },
        }
        self.assertDictEqual(result, expected)


class TestCorporationRoleFilter(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_entities()
        cls.character_1001 = create_memberaudit_character(1001)
        cls.user_1 = cls.character_1001.character_ownership.user
        cls.corporation_2001 = EveCorporationInfo.objects.get(corporation_id=2001)
        cls.corporation_2101 = EveCorporationInfo.objects.get(corporation_id=2101)

    def test_should_return_name(self):
        # given
        my_filter = create_corporation_role_filter(corporations=[])
        # when/then
        self.assertTrue(my_filter.name)

    def test_should_return_false_when_user_does_not_have_role(self):
        # given
        my_filter = create_corporation_role_filter(corporations=[self.corporation_2001])
        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_true_when_user_has_character_with_role_in_corp(self):
        # given
        my_filter = create_corporation_role_filter(
            corporations=[self.corporation_2001], role=CharacterRole.Role.DIRECTOR
        )
        create_character_role(
            character=self.character_1001, role=CharacterRole.Role.DIRECTOR
        )
        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))

    def test_should_return_false_when_user_role_is_not_universal(self):
        # given
        my_filter = create_corporation_role_filter(
            corporations=[self.corporation_2001], role=CharacterRole.Role.DIRECTOR
        )
        create_character_role(
            character=self.character_1001,
            role=CharacterRole.Role.DIRECTOR,
            location=CharacterRole.Location.OTHER,
        )
        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_false_when_character_with_role_is_in_wrong_corp(self):
        # given
        my_filter = create_corporation_role_filter(
            corporations=[self.corporation_2101], role=CharacterRole.Role.DIRECTOR
        )
        my_filter.corporations.add(self.corporation_2101)
        create_character_role(
            character=self.character_1001,
            role=CharacterRole.Role.DIRECTOR,
        )
        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_false_character_with_role_owned_by_other_user(self):
        # given
        my_filter = create_corporation_role_filter(
            corporations=[self.corporation_2101], role=CharacterRole.Role.DIRECTOR
        )
        character_1002 = create_memberaudit_character(1002)
        character_1002.character_ownership.user
        create_character_role(
            character=character_1002,
            role=CharacterRole.Role.DIRECTOR,
        )
        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_false_when_character_with_role_is_not_main(self):
        # given filter for mains only
        my_filter = create_corporation_role_filter(
            corporations=[self.corporation_2001],
            role=CharacterRole.Role.DIRECTOR,
            include_alts=False,
        )
        # and character has role, but is not main
        character_1002 = add_memberaudit_character_to_user(self.user_1, 1002)
        create_character_role(
            character=character_1002, role=CharacterRole.Role.DIRECTOR
        )
        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_true_when_character_with_role_is_not_main_but_allowed(self):
        # given filter for mains only
        my_filter = create_corporation_role_filter(
            corporations=[self.corporation_2001],
            role=CharacterRole.Role.DIRECTOR,
            include_alts=True,
        )
        # and character has role, but is not main
        character_1002 = add_memberaudit_character_to_user(self.user_1, 1002)
        create_character_role(
            character=character_1002, role=CharacterRole.Role.DIRECTOR
        )
        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))

    def test_should_return_audit_data_for_three_users_and_no_alts(self):
        # given
        my_filter = create_corporation_role_filter(
            corporations=[self.corporation_2001, self.corporation_2101],
            role=CharacterRole.Role.DIRECTOR,
            include_alts=False,
        )
        create_character_role(
            character=self.character_1001, role=CharacterRole.Role.DIRECTOR
        )

        character_1101 = create_memberaudit_character(1101)
        user_2 = character_1101.character_ownership.user
        character_1102 = add_memberaudit_character_to_user(user_2, 1102)
        create_character_role(
            character=character_1102, role=CharacterRole.Role.DIRECTOR
        )

        user_3, _ = create_user_from_evecharacter_with_access(1003)

        # when
        users = make_user_queryset(self.user_1, user_2, user_3)
        result = my_filter.audit_filter(users)

        # then
        self.assertEqual(len(result), 3)
        self.assertTrue(result[self.user_1.pk]["check"])
        self.assertFalse(result[user_2.pk]["check"])
        self.assertFalse(result[user_3.pk]["check"])

    def test_should_return_audit_data_for_two_users_with_alts(self):
        # given
        my_filter = create_corporation_role_filter(
            corporations=[self.corporation_2001, self.corporation_2101],
            role=CharacterRole.Role.DIRECTOR,
            include_alts=True,
        )
        create_character_role(
            character=self.character_1001, role=CharacterRole.Role.DIRECTOR
        )
        character_1002 = add_memberaudit_character_to_user(self.user_1, 1002)
        create_character_role(
            character=character_1002, role=CharacterRole.Role.DIRECTOR
        )

        character_1101 = create_memberaudit_character(1101)
        user_2 = character_1101.character_ownership.user
        create_character_role(
            character=character_1101, role=CharacterRole.Role.DIRECTOR
        )

        user_3, _ = create_user_from_evecharacter_with_access(1102)

        # when
        users = make_user_queryset(self.user_1, user_2, user_3)
        result = my_filter.audit_filter(users)

        # then
        self.assertEqual(len(result), 3)
        self.assertDictEqual(
            result[self.user_1.id],
            {"message": "Bruce Wayne, Clark Kent", "check": True},
        )
        self.assertDictEqual(
            result[user_2.id], {"message": "Lex Luther", "check": True}
        )
        self.assertDictEqual(
            result[user_3.id], {"message": "No matching character", "check": False}
        )


class TestCorporationTitleFilter(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_entities()
        cls.character_1001 = create_memberaudit_character(1001)
        cls.user_1 = cls.character_1001.character_ownership.user
        cls.corporation_2001 = EveCorporationInfo.objects.get(corporation_id=2001)
        cls.corporation_2101 = EveCorporationInfo.objects.get(corporation_id=2101)

    def test_should_return_name(self):
        # given
        my_filter = create_corporation_title_filter(corporations=[])

        # when/then
        self.assertTrue(my_filter.name)

    def test_should_return_false_when_user_does_not_have_title(self):
        # given
        my_filter = create_corporation_title_filter(corporations=[])

        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_true_when_user_has_character_with_title_in_corp(self):
        # given
        my_filter = create_corporation_title_filter(
            corporations=[self.corporation_2001], title="Alpha"
        )
        create_character_title(character=self.character_1001, name="Alpha")

        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))

    def test_should_not_return_true_when_user_has_character_with_title_but_corp_not_defined(
        self,
    ):
        # given
        my_filter = create_corporation_title_filter(corporations=[], title="Alpha")
        create_character_title(character=self.character_1001, name="Alpha")

        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_false_when_character_with_title_is_in_wrong_corp(self):
        # given
        my_filter = create_corporation_title_filter(
            corporations=[self.corporation_2101], title="Alpha"
        )
        create_character_title(character=self.character_1001, name="Alpha")

        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_false_character_with_title_is_owned_by_other_user(self):
        # given
        my_filter = create_corporation_title_filter(
            corporations=[self.corporation_2001], title="Alpha"
        )
        character_1002 = create_memberaudit_character(1002)
        character_1002.character_ownership.user
        create_character_title(character=character_1002, name="Alpha")

        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_false_when_character_with_title_is_not_main(self):
        # given filter for mains only
        my_filter = create_corporation_title_filter(
            corporations=[self.corporation_2001], title="Alpha", include_alts=False
        )
        # and owned character has title, but is not main
        character_1002 = add_memberaudit_character_to_user(self.user_1, 1002)
        create_character_title(character=character_1002, name="Alpha")

        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_true_when_character_with_title_is_not_main_but_allowed(self):
        # given filter for mains only
        my_filter = create_corporation_title_filter(
            corporations=[self.corporation_2001], title="Alpha", include_alts=True
        )

        # and character has title, but is not main
        character_1002 = add_memberaudit_character_to_user(self.user_1, 1002)
        create_character_title(character=character_1002, name="Alpha")

        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))

    def test_should_return_audit_data_for_users_and_mains_only(self):
        # given
        my_filter = create_corporation_title_filter(
            corporations=[self.corporation_2001, self.corporation_2101],
            title="Alpha",
            include_alts=False,
        )
        create_character_title(character=self.character_1001, name="Alpha")
        character_1002 = add_memberaudit_character_to_user(self.user_1, 1002)
        create_character_title(character=character_1002, name="Alpha")

        character_1101 = create_memberaudit_character(1101)
        user_2 = character_1101.character_ownership.user
        character_1102 = add_memberaudit_character_to_user(user_2, 1102)
        create_character_title(character=character_1102, name="Alpha")

        user_3, _ = create_user_from_evecharacter_with_access(1104)

        # when
        users = make_user_queryset(self.user_1, user_2, user_3)
        result = my_filter.audit_filter(users)

        # then
        self.assertEqual(len(result), 3)
        self.assertDictEqual(
            result[self.user_1.id], {"message": "Bruce Wayne", "check": True}
        )
        self.assertFalse(result[user_2.id]["check"])
        self.assertFalse(result[user_3.id]["check"])

    def test_should_return_audit_data_for_three_users_and_including_alts(self):
        # given
        my_filter = create_corporation_title_filter(
            corporations=[self.corporation_2001, self.corporation_2101],
            title="Alpha",
            include_alts=True,
        )
        character_1002 = add_memberaudit_character_to_user(self.user_1, 1002)
        create_character_title(character=character_1002, name="Alpha")

        character_1101 = create_memberaudit_character(1101)
        user_2 = character_1101.character_ownership.user
        create_character_title(character=character_1101, name="Alpha")

        user_3, _ = create_user_from_evecharacter_with_access(1104)

        # when
        users = make_user_queryset(self.user_1, user_2, user_3)
        result = my_filter.audit_filter(users)

        # then
        self.assertEqual(len(result), 3)
        self.assertTrue(result[self.user_1.id]["check"])
        self.assertTrue(result[user_2.id]["check"])
        self.assertFalse(result[user_3.id]["check"])


class TestSkillSetFilterBase(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_entities()
        load_eveuniverse()
        # user with a main and an alt
        cls.character_1001 = create_memberaudit_character(1001)
        cls.user_1 = cls.character_1001.character_ownership.user
        cls.character_1002 = add_memberaudit_character_to_user(cls.user_1, 1002)
        # amarr carrier skill set
        cls.amarr_carrier_skill_type = EveType.objects.get(name="Amarr Carrier")
        cls.amarr_carrier_skill_set = create_skill_set()
        cls.amarr_carrier_skill_set_skill = create_skill_set_skill(
            skill_set=cls.amarr_carrier_skill_set,
            eve_type=cls.amarr_carrier_skill_type,
            required_level=3,
            recommended_level=5,
        )
        # caldari carrier skill set
        cls.caldari_carrier_skill_type = EveType.objects.get(name="Caldari Carrier")
        cls.caldari_carrier_skill_set = create_skill_set()
        cls.caldari_carrier_skill_set_skill = create_skill_set_skill(
            skill_set=cls.caldari_carrier_skill_set,
            eve_type=cls.caldari_carrier_skill_type,
            required_level=3,
            recommended_level=5,
        )


class TestSkillSetFilterProcessFilter(TestSkillSetFilterBase):
    def test_should_return_name(self):
        # given
        my_filter = SkillSetFilter.objects.create()
        # when/then
        self.assertTrue(my_filter.name)

    def test_should_return_false_when_user_does_not_have_skill_set_check(self):
        # given
        my_filter = SkillSetFilter.objects.create()
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_false_when_user_did_not_pass_skill_set_check(self):
        # given
        my_filter = SkillSetFilter.objects.create()
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        skill_set_check = create_character_skill_set_check(
            character=self.character_1001, skill_set=self.amarr_carrier_skill_set
        )
        skill_set_check.failed_required_skills.add(self.amarr_carrier_skill_set_skill)
        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_true_when_user_passes_skill_set(self):
        # given
        my_filter = SkillSetFilter.objects.create()
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            character=self.character_1001, skill_set=self.amarr_carrier_skill_set
        )
        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))

    def test_should_return_true_when_user_passes_skill_set_except_recommended_skills(
        self,
    ):
        # given
        my_filter = SkillSetFilter.objects.create()
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        skill_set_check = create_character_skill_set_check(
            character=self.character_1001, skill_set=self.amarr_carrier_skill_set
        )
        skill_set_check.failed_recommended_skills.add(
            self.amarr_carrier_skill_set_skill
        )
        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))

    def test_should_return_false_when_character_is_main_but_alt_required(self):
        # given
        my_filter = SkillSetFilter.objects.create(
            character_type=SkillSetFilter.CharacterType.ALTS_ONLY
        )
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            character=self.character_1001, skill_set=self.amarr_carrier_skill_set
        )
        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_false_when_character_is_main_and_main_required(self):
        # given
        my_filter = SkillSetFilter.objects.create(
            character_type=SkillSetFilter.CharacterType.MAINS_ONLY
        )
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            character=self.character_1001, skill_set=self.amarr_carrier_skill_set
        )
        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))

    def test_should_return_true_when_character_is_main_and_any_allowed(self):
        # given
        my_filter = SkillSetFilter.objects.create(
            character_type=SkillSetFilter.CharacterType.ANY
        )
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            character=self.character_1001, skill_set=self.amarr_carrier_skill_set
        )
        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))

    def test_should_return_false_when_character_is_alt_but_main_required(self):
        # given
        my_filter = SkillSetFilter.objects.create(
            character_type=SkillSetFilter.CharacterType.MAINS_ONLY
        )
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            self.character_1002, skill_set=self.amarr_carrier_skill_set
        )
        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1))

    def test_should_return_true_when_character_is_alt_and_any_allowed(self):
        # given
        my_filter = SkillSetFilter.objects.create(
            character_type=SkillSetFilter.CharacterType.ANY
        )
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            self.character_1002, skill_set=self.amarr_carrier_skill_set
        )
        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))

    def test_should_return_true_when_character_is_alt_and_alt_required(self):
        # given
        my_filter = SkillSetFilter.objects.create(
            character_type=SkillSetFilter.CharacterType.ALTS_ONLY
        )
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            self.character_1002, skill_set=self.amarr_carrier_skill_set
        )
        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1))


class TestSkillSetFilterAuditFilter(TestSkillSetFilterBase):
    def test_should_return_audit_data_with_several_users(self):
        # given
        my_filter = SkillSetFilter.objects.create()
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            character=self.character_1001, skill_set=self.amarr_carrier_skill_set
        )
        create_character_skill_set_check(
            character=self.character_1002, skill_set=self.amarr_carrier_skill_set
        )

        character_1101 = create_memberaudit_character(1101)
        user_2 = character_1101.character_ownership.user
        create_character_skill_set_check(
            character=character_1101, skill_set=self.amarr_carrier_skill_set
        )

        user_3, _ = create_user_from_evecharacter_with_access(1102)

        # when
        users = make_user_queryset(self.user_1, user_2, user_3)
        result = my_filter.audit_filter(users)

        # then
        self.assertEqual(len(result), 3)
        self.assertTrue(result[self.user_1.id]["check"])
        self.assertTrue(result[user_2.id]["check"])
        self.assertFalse(result[user_3.id]["check"])

    def test_should_return_audit_data_when_character_is_main_but_alt_required(self):
        # given
        my_filter = SkillSetFilter.objects.create(
            character_type=SkillSetFilter.CharacterType.ALTS_ONLY
        )
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            character=self.character_1001, skill_set=self.amarr_carrier_skill_set
        )

        # when
        users = make_user_queryset(self.user_1)
        result = my_filter.audit_filter(users)

        # then
        self.assertEqual(len(result), 1)
        self.assertFalse(result[self.user_1.id]["check"])

    def test_should_return_audit_data_when_character_is_main_and_main_required(self):
        # given
        my_filter = SkillSetFilter.objects.create(
            character_type=SkillSetFilter.CharacterType.MAINS_ONLY
        )
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            character=self.character_1001, skill_set=self.amarr_carrier_skill_set
        )

        # when
        users = make_user_queryset(self.user_1)
        result = my_filter.audit_filter(users)

        # then
        expected = {self.user_1.id: {"check": True, "message": "Bruce Wayne"}}
        self.assertDictEqual(result, expected)

    def test_should_return_audit_data_when_character_is_main_and_any_allowed(self):
        # given
        my_filter = SkillSetFilter.objects.create(
            character_type=SkillSetFilter.CharacterType.ANY
        )
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            character=self.character_1001, skill_set=self.amarr_carrier_skill_set
        )
        # when
        users = make_user_queryset(self.user_1)
        result = my_filter.audit_filter(users)

        # then
        expected = {self.user_1.id: {"check": True, "message": "Bruce Wayne"}}
        self.assertDictEqual(result, expected)

    def test_should_return_audit_data_when_character_is_alt_but_main_required(self):
        # given
        my_filter = SkillSetFilter.objects.create(
            character_type=SkillSetFilter.CharacterType.MAINS_ONLY
        )
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            self.character_1002, skill_set=self.amarr_carrier_skill_set
        )
        # when
        users = make_user_queryset(self.user_1)
        result = my_filter.audit_filter(users)

        # then
        self.assertEqual(len(result), 1)
        self.assertFalse(result[self.user_1.id]["check"])

    def test_should_return_audit_data_when_character_is_alt_and_any_allowed(self):
        # given
        my_filter = SkillSetFilter.objects.create(
            character_type=SkillSetFilter.CharacterType.ANY
        )
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            self.character_1002, skill_set=self.amarr_carrier_skill_set
        )

        # when
        users = make_user_queryset(self.user_1)
        result = my_filter.audit_filter(users)

        # then
        expected = {self.user_1.id: {"check": True, "message": "Clark Kent"}}
        self.assertDictEqual(result, expected)

    def test_should_return_audit_data_when_character_is_alt_and_alt_required(self):
        # given
        my_filter = SkillSetFilter.objects.create(
            character_type=SkillSetFilter.CharacterType.ANY
        )
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            self.character_1002, skill_set=self.amarr_carrier_skill_set
        )

        # when
        users = make_user_queryset(self.user_1)
        result = my_filter.audit_filter(users)

        # then
        expected = {self.user_1.id: {"check": True, "message": "Clark Kent"}}
        self.assertDictEqual(result, expected)

    def test_should_default_to_any_as_character_type(self):
        # given
        my_filter = SkillSetFilter.objects.create(character_type="")
        my_filter.skill_sets.add(
            self.amarr_carrier_skill_set, self.caldari_carrier_skill_set
        )
        create_character_skill_set_check(
            self.character_1002, skill_set=self.amarr_carrier_skill_set
        )

        self.assertEqual(my_filter.character_type, SkillSetFilter.CharacterType.ANY)


class TestTimeInCorporationFilter(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_entities()
        cls.character = create_memberaudit_character(1001)
        cls.user_1001 = cls.character.character_ownership.user
        cls.corporation_2001 = EveEntity.objects.get(id=2001)
        cls.corporation_2002 = EveEntity.objects.get(id=2002)

    def test_should_return_name(self):
        # given
        my_filter = create_time_in_corporation_filter()

        # when/then
        self.assertTrue(my_filter.name)

    def test_should_return_true_when_main_membership_was_long_enough(self):
        # given
        create_character_corporation_history(
            character=self.character,
            corporation=self.corporation_2001,
            start_date=now() - dt.timedelta(days=30),
        )
        my_filter = create_time_in_corporation_filter(minimum_days=30)

        # when/then
        self.assertTrue(my_filter.process_filter(self.user_1001))

    def test_should_return_false_when_main_membership_was_not_long_enough(self):
        # given
        create_character_corporation_history(
            character=self.character,
            corporation=self.corporation_2001,
            start_date=now() - dt.timedelta(days=29),
        )
        my_filter = create_time_in_corporation_filter(minimum_days=30)

        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1001))

    def test_should_return_false_when_no_membership_data_for_main(self):
        # given
        my_filter = create_time_in_corporation_filter(minimum_days=30)

        # when/then
        self.assertFalse(my_filter.process_filter(self.user_1001))

    def test_should_return_false_when_user_has_no_memberaudit_character(self):
        # given
        my_filter = create_time_in_corporation_filter(minimum_days=30)
        user, _ = create_user_from_evecharacter_with_access(1002)

        # when/then
        self.assertFalse(my_filter.process_filter(user))

    def test_should_return_audit_data_with_one_user_passing_and_one_not_passing(self):
        # given
        my_filter = create_time_in_corporation_filter(minimum_days=30)
        create_character_corporation_history(
            character=self.character,
            corporation=self.corporation_2001,
            start_date=now() - dt.timedelta(days=30),
        )
        character_1101 = create_memberaudit_character(1101)
        user_1101 = character_1101.user
        create_character_corporation_history(
            record_id=1,
            character=character_1101,
            corporation=self.corporation_2002,
            start_date=now() - dt.timedelta(days=100),
        )
        create_character_corporation_history(
            record_id=2,
            character=character_1101,
            corporation=self.corporation_2001,
            start_date=now() - dt.timedelta(days=29),
        )
        users = User.objects.filter(pk__in=[self.user_1001.pk, user_1101.pk])
        # when
        result = my_filter.audit_filter(users)
        # then
        expected = {
            self.user_1001.id: {"message": "30 days", "check": True},
            user_1101.id: {"message": "29 days", "check": False},
        }
        self.assertDictEqual(dict(result), expected)


def make_user_queryset(*users):
    params = {"pk__in": [obj.pk for obj in users]}
    return User.objects.filter(**params)



# Generated by Django 4.0.10 on 2023-10-12 18:12

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("memberaudit", "0013_add_corporation_titles"),
        ("eveonline", "0017_alliance_and_corp_names_are_not_unique"),
        (
            "memberaudit_securegroups",
            "0002_alter_skillpointfilter_skill_point_threshold_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="skillsetfilter",
            name="character_type",
            field=models.CharField(
                blank=True,
                choices=[("AN", "Any"), ("MO", "Mains only"), ("AO", "Alts only")],
                default="AN",
                help_text="Specify the type of character that needs to have the skill set.",
                max_length=2,
            ),
        ),
        migrations.AlterField(
            model_name="skillsetfilter",
            name="skill_sets",
            field=models.ManyToManyField(
                help_text="Users must have a character who possess all of the skills in <strong>one</strong> of the selected skill sets.",
                to="memberaudit.skillset",
            ),
        ),
        migrations.CreateModel(
            name="CorporationRoleFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "description",
                    models.CharField(
                        help_text="The filter description that is shown to end users.",
                        max_length=500,
                    ),
                ),
                (
                    "role",
                    models.CharField(
                        choices=[
                            ("AT1", "account take 1"),
                            ("AT2", "account take 2"),
                            ("AT3", "account take 3"),
                            ("AT4", "account take 4"),
                            ("AT5", "account take 5"),
                            ("AT6", "account take 6"),
                            ("AT7", "account take 7"),
                            ("ACT", "accountant"),
                            ("AUD", "auditor"),
                            ("COM", "communications officer"),
                            ("CEQ", "config equipment"),
                            ("CSE", "config starbase equipment"),
                            ("CT1", "container take 1"),
                            ("CT2", "container take 2"),
                            ("CT3", "container take 3"),
                            ("CT4", "container take 4"),
                            ("CT5", "container take 5"),
                            ("CT6", "container take 6"),
                            ("CT7", "container take 7"),
                            ("CMG", "contract manager"),
                            ("DPL", "diplomat"),
                            ("DRT", "director"),
                            ("FCM", "factory manager"),
                            ("FTM", "fitting manager"),
                            ("HQ1", "hangar query 1"),
                            ("HQ2", "hangar query 2"),
                            ("HQ3", "hangar query 3"),
                            ("HQ4", "hangar query 4"),
                            ("HQ5", "hangar query 5"),
                            ("HQ6", "hangar query 6"),
                            ("HQ7", "hangar query 7"),
                            ("HT1", "hangar take 1"),
                            ("HT2", "hangar take 2"),
                            ("HT3", "hangar take 3"),
                            ("HT4", "hangar take 4"),
                            ("HT5", "hangar take 5"),
                            ("HT6", "hangar take 6"),
                            ("HT7", "hangar take 7"),
                            ("JAC", "junior accountant"),
                            ("PSM", "personnel manager"),
                            ("RFF", "rent factory facility"),
                            ("RFC", "rent office"),
                            ("RRF", "rent research facility"),
                            ("SCO", "security officer"),
                            ("SPM", "skill plan manager"),
                            ("SDO", "starbase defense operator"),
                            ("SFT", "starbase fuel technician"),
                            ("STM", "station manager"),
                            ("TRD", "trader"),
                        ],
                        db_index=True,
                        help_text="User must have a character with this role.",
                        max_length=3,
                    ),
                ),
                (
                    "include_alts",
                    models.BooleanField(
                        default=False,
                        help_text="When True, the filter will also include the users alt-characters.",
                    ),
                ),
                (
                    "corporations",
                    models.ManyToManyField(
                        help_text="The character with the role must be in one of these corporations.",
                        related_name="+",
                        to="eveonline.evecorporationinfo",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]
